module(...,package.seeall)

local msgQuene = {}

function procMsg(mqttc)
	while #msgQuene>0 do
		local outMsg = table.remove(msgQuene,1)
		local result = mqttc:publish(outMsg.t,outMsg.p,outMsg.q)
		if outMsg.user then outMsg.user.cb(result,outMsg.user.para) end
		if not result then return end
	end
	return true
end

function insertOutMsg(topic,payload,qos,user)
	table.insert(msgQuene,{t=topic,p=payload,q=0,user=user})
end

function waitForSend()
	return #msgQuene > 0
end

local shkcnt = 0

--[[
函数名：getstatus
功能  ：获取需要状态
参数  ：无
返回值：状态
]]
local function getstatus()
	local t = {}

	t.shake = (shkcnt > 0) and 1 or 0
	shkcnt = 0
	t.charger = chg.getcharger() and 1 or 0
	t.acc = acc.getflag() and 1 or 0
	t.gps = gps.isOpen() and 1 or 0
	t.sleep = pm.isSleep() and 1 or 0
	t.volt = chg.getvolt()	
	t.fly = 0
	t.poweroff = 0
	t.rest = 0
	return t
end

--[[
函数名：getgps
功能  ：获取GPS状态
参数  ：无
返回值：GPS经纬度，定位状态，速度
]]
local function getgps()
	local t = {}
	local Location = gps.getLocation()
	print("getgps:",Location.lngType,Location.lng,Location.latType,Location.lat,gps.getCourse(),gps.getSpeed())
	t.fix = gps.isFix()
	if gps.isFix() then
		t.lng,t.lat = Location.lng,Location.lat
	else
		t.lng,t.lat = manage.getlastgps()
	end
	t.fix = true
	t.lng,t.lat = t.lng or "",t.lat or ""
	t.cog = gps.getCourse()
	t.spd = gps.getSpeed()

	return t
end

--[[
函数名：getgpstat
功能  ：获取GPS可见卫星个数
参数  ：无
返回值：GPS可见卫星个数
]]
local function getgpstat()
	local t = {}
	t.satenum = gps.getViewedSateCnt()
	return t
end

local tGet = {
	["VERSION"] = function() return _G.VERSION end,
	PROJECTID = function() return _G.PRJID end,
	PROJECT = function() return _G.PROJECT end,
	HEART = function() return nvm.get("heart") end,
	IMEI = misc.getImei,
	IMSI = sim.getImsi,
	ICCID = sim.getIccid,
	GPS = getgps,
	GPSTAT = getgpstat,
	RSSI = net.getRssi,
	CELLINFOEXT = net.getCellInfoExt,
	TA = net.getTa,
	STATUS = getstatus,
	GPSLEEP = function() return nvm.get("gpsleep") end,
	PARAVER = function() return nvm.get("paraver") end,
}
local function getf(id)
	assert(tGet[id] ~= nil,"getf nil id:" .. id)
	return tGet[id]()
end

function enTopic(t)
	return "/v2/device/"..tGet["IMEI"]().."/"..t
end

local function heartReport()
	insertOutMsg(enTopic("devdata"),protoair.pack(protoair.HEART),0)
	sys.timerStart(heartReport,nvm.get("heart")*1000)
end

function locReport(typ,user)
	insertOutMsg(enTopic("devdata"),protoair.pack(protoair.GPSLBS1,typ),0,user)
	sys.timerStart(heartReport,nvm.get("heart")*1000)
end

function prelocReport()
	print("prelocReport",shkcnt,gps.isFix())
	if shkcnt > 0 and gps.isFix() then
		locReport()	
	end
	sys.timerStart(prelocReport,nvm.get("rptfreq")*1000)
end

function init()
	insertOutMsg(enTopic("devdata"),protoair.pack(protoair.INFO),0)
	--locReport()
	heartReport()
end



local devpararpting

function getdevpara(idx)
	devpararpting = true
	local s = split.getfdata(nvm.paraname,idx,2048)
	log.info("linkout.getdevpara",idx,string.len(s))
	if not s or s=="" then
		devpararpting = false
	end
	return s
end

local function devpararptreq(result,idx)
	log.info("linkout.devpararptreq",devpararpting,idx)
	if not devpararpting or idx then
		local s = getdevpara(idx or 1)
		if s and s~="" then insertOutMsg(enTopic("devpararpt"),protoair.pack(protoair.RPTPARA,pack.pack("bA",protoair.LOG,s)),1,{cb=devpararptreq,para=(idx or 1)+1}) end
	end
end

local slen,ssub,schar = string.len,string.sub,string.char

function unInit()
	sys.timerStop(locReport)
	sys.timerStop(heartReport)
	while #msgQuene>0 do
		local outMsg = table.remove(msgQuene,1)
		if outMsg.user then outMsg.user.cb(false,outMsg.user.para) end
	end
	devpararpting = nil
end

local function shkind()
	shkcnt = shkcnt + 1
	return true
end

local firstgps
local function gpstateind(evt)
	print("gpstateind",evt,firstgps,rtos.poweron_reason())
	if evt == "LOCATION_SUCCESS" and not firstgps then
		if rtos.poweron_reason() == rtos.POWERON_KEY or  rtos.poweron_reason() == rtos.POWERON_CHARGER then
			locReport()
			firstgps = true
		else
			prelocReport()
		end
	end
	return true
end

--sys.subscribe("DEVPARA_RPT_REQ",devpararptreq)
sys.subscribe("SHKCNT_VALIDSHK_IND",shkind)
sys.subscribe("GPS_STATE",gpstateind)
protoair.reget(getf)
